unit Tablet;

interface

uses
  Windows, SysUtils, Messages, Classes, Forms, WintabConsts;

type
  TAxis = record
  	Min: Integer;
    Max: Integer;
  	Units: Word;
  	Resolution: Integer;
  end;

  TLogContext = record
    Name: string[39];
    Options: Integer;
    Status: Integer;
    Locks: Integer;
    MsgBase: Integer;
    Device: Integer;
    PktRate: Integer;
    PktData: Integer;
    PktMode: Integer;
    MoveMask: Integer;
    BtnDnMask: Integer;
    BtnUpMask: Integer;
    InOrgX: Integer;
    InOrgY: Integer;
    InOrgZ: Integer;
    InExtX: Integer;
    InExtY: Integer;
    InExtZ: Integer;
    OutOrgX: Integer;
    OutOrgY: Integer;
    OutOrgZ: Integer;
    OutExtX: Integer;
    OutExtY: Integer;
    OutExtZ: Integer;
    SensX: Integer;
    SensY: Integer;
    SensZ: Integer;
    SysMode: Integer;
    SysOrgX: Integer;
    SysOrgY: Integer;
    SysExtX: Integer;
    SysExtY: Integer;
    SysSensX: Integer;
    SysSensY: Integer;
  end;

  TString = array[0..100] of char;
  TPacketEvent = procedure(Sender: TObject; PacketNumber, ContextHandle: Integer;
    pPacket: Pointer) of object;
  TProximityEvent = procedure(Sender: TObject; ContextHandle: Integer;
    Entering: Boolean) of object;
  TInfoChangedEvent = procedure(Sender: TObject; Manager: THandle;
    Category, Index: Word) of object;
  TContextEvent =  procedure(Sender: TObject; ContextHandle, StatusFlags: Integer) of object;

  TTablet = class(TComponent)
  private
    FOnPacket: TPacketEvent;
    FOnCursorChanged: TPacketEvent;
    FOnProximity: TProximityEvent;
    FOnInfoChanged: TInfoChangedEvent;
    FOnOpenContext: TContextEvent;
    FOnCloseContext: TContextEvent;
    FOnOverlapContext: TContextEvent;
    FOnUpdateContext: TContextEvent;
    FOldWindowProc,
    FNewWindowProc: Pointer;
    FDeviceName: string;
    FPacketSize: Word;
    FPresent: Boolean;
    FEnabled: Boolean;
    FPPacket: Pointer;
    FContextHandle: THandle;
    FContext: TLogContext;
    procedure NewWindowProc(var Msg: TMessage);
    procedure SetEnabled(Value: Boolean);
  protected
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Initialize;
    function GetContext: Boolean;
    function SetContext: Boolean;
    property Context: TLogContext read FContext write FContext;
    property ContextHandle: THandle read FContextHandle write FContextHandle;
    property Present: Boolean read FPresent write FPresent;
  published
    property Enabled: Boolean read FEnabled write SetEnabled;
    property PacketSize: Word read FPacketSize write FPacketSize;
    property DeviceName: string read FDeviceName write FDeviceName;
    property OnContextClose: TContextEvent read FOnCloseContext write FOnCloseContext;
    property OnContextOpen: TContextEvent read FOnOpenContext write FOnOpenContext;
    property OnContextOverlap: TContextEvent read FOnOverlapContext write FOnOverlapContext;
    property OnContextUpdate: TContextEvent read FOnUpdateContext write FOnUpdateContext;
    property OnCursorChanged: TPacketEvent read FOnCursorChanged write FOnCursorChanged;
    property OnInfoChanged: TInfoChangedEvent read FOnInfoChanged write FOnInfoChanged;
    property OnPacket: TPacketEvent read FOnPacket write FOnPacket;
    property OnProximity: TProximityEvent read FOnProximity write FOnProximity;
  end;

  procedure Register;

//Aliased functions (Can change these to unicode by changing the A to W in Alias name)
    function WTInfo(wCategory: Integer; nIndex: Integer;
      lpOutput: Pointer): Integer; stdcall; external 'wintab32.dll' name 'WTInfoA';
    //Used to read dispidious pieces of information about the tablet.

    function WTOpen(hwnd: Integer; var LPLogContext: TLogContext; fEnable: Boolean):
     Integer; stdcall; external 'wintab32.dll' name 'WTOpenA';  //Should be: HCTX
    //Used to begin accessing the Tablet.

    function WTGet(hCtx: Integer; var LPLogContext: TLogContext): Integer;
      stdcall; external 'wintab32.dll' name 'WTGetA';  //Should be: BOOL
    //Fills the supplied structure with the current context attributes for the passed handle.

    function WTSet(hCtx: Integer; var LPLogContext: TLogContext): Integer;
      stdcall; external 'wintab32.dll' name 'WTSetA';  //Should be: BOOL
    //Allows some of the context//s attributes to be changed on the fly.

//Basic functions
    function WTClose(hCtx: Integer): Integer; stdcall; external 'wintab32.dll';  //Should be: BOOL
    //Used to end accessing the Tablet.

    function WTPacketsGet(hCtx: Integer; cMaxPackets: Integer; lpPkts: Pointer): Integer;
      stdcall; external 'wintab32.dll';
    //Used to poll the Tablet for input.

    function WTPacket(hCtx: Integer; wSerial: Integer; lpPkts: Pointer): Integer
      ; stdcall; external 'wintab32.dll';  //Should be: BOOL
    //Similar to WTPacketsGet but is used in a window function.

//Visibility functions
    function WTEnable(hCtx: Integer; fEnable: Integer): Integer
      ; stdcall; external 'wintab32.dll';
    //Enables and Disables a Tablet Context; temporarily turning on or off the processing of packets.

    function WTOverlap(hCtx: Integer; fToTop: Boolean): Integer;
      stdcall; external 'wintab32.dll';
    //Sends a tablet context to the top or bottom of the order of overlapping tablet contexts.

//Context Editing functions
    function WTConfig(hCtx: Integer; hwnd:Integer): Integer;
      stdcall; external 'wintab32.dll';
    //Used to call a requestor which aids in configuring the Tablet

    function WTExtGet(hCtx: Integer; wExt: Integer; lpData: Pointer): Integer;
      stdcall; external 'wintab32.dll';

    function WTExtSet(hCtx: Integer; wExt: Integer; lpData: Pointer): Integer;
      stdcall; external 'wintab32.dll';

    function WTSave(hCtx: Integer; lpSaveInfo: Pointer): Integer;
      stdcall; external 'wintab32.dll';
    //Fills the supplied buffer with binary save information that can be used to restore the equivalent context in a subsequent Windows session.

    function WTRestore(hwnd: Integer; lpSaveInfo: Pointer; fEnable: Integer): Integer;
      stdcall; external 'wintab32.dll';
    //Creates a tablet context from the save information returned from the WTSave function.

//Advanced Packet and Queue functions
    function WTPacketsPeek(hCtx: Integer; cMaxPackets: Integer; lpPkts: Pointer): Integer;
      stdcall; external 'wintab32.dll';
    function WTDataGet(hCtx: Integer; wBegin: Integer; wEnding: Integer; cMaxPackets: Integer; lpPkts: Pointer; lpNPkts: Pointer): Integer;
      stdcall; external 'wintab32.dll';
    function WTDataPeek(hCtx: Integer; wBegin: Integer; wEnding: Integer; cMaxPackets: Integer; lpPkts: Pointer; lpNPkts: Pointer): Integer
      ; stdcall; external 'wintab32.dll';
    function WTQueuePacketsEx(hCtx: Integer; var lpOld: Integer; var lpNew: Integer): Integer
      ; stdcall; external 'wintab32.dll';
    //Returns the serial numbers of the oldest and newest packets currently in the queue.

    function WTQQueueSizeGet(hCtx:Integer): Integer; stdcall; external 'wintab32.dll';
    function WTQQueueSizeSet(hCtx:Integer; nPkts: Integer): Integer
      ; stdcall; external 'wintab32.dll';

//Manager functions

    //Manager Handle functions
        function WTMgrOpen(hwnd:Integer; wMsgBase: Integer):Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrClose(hMgr:Integer): Boolean; stdcall; external 'wintab32.dll';

    //Manager Context functions
        function WTMgrContextEnum(hMgr:Integer; lpEnumFunc: Pointer; lParam: Pointer): Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrContextOwner(hMgr:Integer; hCtx:Integer):Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrDefContext(hMgr:Integer; fSystem: Integer):Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrDefContextEx(hMgr:Integer; wDevice: Integer; fSystem: Integer):Integer
          ; stdcall; external 'wintab32.dll';

    //Manager Configuration functions
        function WTMgrDeviceConfig(hMgr:Integer; var wDevice: Integer; hwnd:Integer): Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrConfigReplaceExA(hMgr:Integer; fInstall: Boolean; lpszModule: Pointer; lpszCfgProc: Pointer): Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrConfigReplaceExW(hMgr:Integer; fInstall: Boolean; lpszModule: Pointer; lpszCfgProc: Pointer): Integer
          ; stdcall; external 'wintab32.dll';

    //Manager Packet Hook functions
        function WTMgrPacketHookExA(hMgr:Integer; nType: Integer; lpszModule: Pointer; lpszHookProc: Pointer):Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrPacketHookExW(hMgr:Integer; nType: Integer; lpszModule: Pointer; lpszHookProc: Pointer):Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrPacketUnHook(hHook:Integer): Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrPacketHookNext(hHook:Integer; nCode: Integer; wParam: Pointer; var lParam: Pointer): Integer
          ; stdcall; external 'wintab32.dll';

    //Manager Preference Data functions
        function WTMgrExt(hMgr:Integer; wExt: Integer; lpData: Pointer): Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrCsrEnable(hMgr:Integer; wCursor: Integer; fEnable: Boolean): Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrCsrButtonMap(hMgr:Integer; wCursor: Integer; lpLogBtns: Pointer; lpSysBtns: Pointer): Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrCsrPressureBtnMarksEx(hMgr:Integer; wCursor: Integer; lpNMarks: Pointer; lpTMarks: Pointer): Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrCsrPressureResponse(hMgr:Integer; wCursor: Integer; lpNResp: Pointer; lpTResp: Pointer): Integer
          ; stdcall; external 'wintab32.dll';
        function WTMgrCsrExt(hMgr:Integer; wCursor: Integer; wExt: Integer; var lpData: Pointer): Integer
          ; stdcall; external 'wintab32.dll';

implementation

procedure Register;
begin
  RegisterComponents('System', [TTablet]);
end;

procedure TTablet.NewWindowProc(var Msg: TMessage);
begin
  with Msg do
  begin
    case Msg of
      WT_CTXCLOSE:
        if Assigned(FOnCloseContext) then
          FOnCloseContext(Self, WParam, LParam);
      WT_CTXOPEN:
        if Assigned(FOnOpenContext) then
          FOnOpenContext(Self, WParam, LParam);
      WT_CTXOVERLAP:
        if Assigned(FOnOverlapContext) then
          FOnOverlapContext(Self, WParam, LParam);
      WT_CTXUPDATE:
        if Assigned(FOnUpdateContext) then
          FOnUpdateContext(Self, WParam, LParam);
      WT_CSRCHANGE:
        if Assigned(FOnCursorChanged) then
          if WTPacket(LParam, WParam, FPPacket) <> 0 then
            FOnCursorChanged(Self, WParam, LParam, FPPacket);
      WT_INFOCHANGE:
        if Assigned(FOnInfoChanged) then
          FOnInfoChanged(Self, WParam, LParamLo, LParamHi);
      WT_PACKET:
        if Assigned(FOnPacket)  then
          if WTPacket(LParam, WParam, FPPacket) <> 0 then
            FOnPacket(Self, WParam, LParam, FPPacket);
      WT_PROXIMITY:
        if Assigned(FOnProximity) then
          FOnProximity(Self, WParam, LParamLo <> 0);
    else

    { Call the old Window procedure to }
    { allow processing of the message. }
      Result := CallWindowProc(FOldWindowProc, Application.Handle, Msg,wParam,lParam);
    end;
  end;
end;

constructor TTablet.Create(AOwner: TComponent);
var
  CharArray: TString;
begin
  inherited Create(AOwner);
  Present := WTInfo(0, 0, nil) <> 0;
  if Present then
  begin
    WTInfo(WTI_DEVICES, DVC_NAME, @CharArray);
    FDeviceName := CharArray;
  end
end;

procedure TTablet.Loaded;
begin
  inherited;
  FNewWindowProc := MakeObjectInstance(NewWindowProc);
  FOldWindowProc := Pointer(SetWindowLong(Application.Handle, GWL_WndProc, longint(FNewWindowProc)));
end;

destructor TTablet.Destroy;
begin
  SetWindowLong(Application.Handle, GWL_WndProc, longint(FOldWindowProc));
  FreeObjectInstance(FNewWindowProc);
  FreeMem(FPPacket, FPacketSize);
  inherited;
end;

procedure TTablet.Initialize;
begin
  WTInfo(WTI_DEFCONTEXT, 0, @Context);    //establish contact and fill context with defaults
end;

function TTablet.SetContext: Boolean;
begin
  Result := WTSet(FContextHandle, FContext) <> 0;
end;

function TTablet.GetContext: Boolean;
begin
  Result := WTGet(FContextHandle, FContext) <> 0;
end;

procedure TTablet.SetEnabled(Value: Boolean);
begin
  if (FEnabled = Value) or (csDesigning in ComponentState) then exit;
  if Value then
  begin
    ReallocMem(FPPacket, FPacketSize + 1);
    FContextHandle := WTOpen(Application.Handle, FContext, True);
    if FContextHandle = 0 then
      raise Exception.Create('Tablet context failed to open.');
    FEnabled := true;
  end
  else
  begin
    WTClose(FContextHandle);
    FEnabled := false;
  end;
end;

end.
